% the granger calculations use the MVGC matlab toolbox
% https://users.sussex.ac.uk/~lionelb/MVGC/html/mvgchelp.html

%%load data structure 
load('exp1and2.mat');
data_d=spd_excl_92; % assign the downsampled shock period data to 'data_d'
clearvars -except 'data_d'; close all


%% calculate granger F for each subject and save in one structure
for i=1:92  %put 1:N
nvars   = 3;    % dem and obs and shock
nobs    = 710-1+1; %710s
ntrials = 1;   % for pairwise analysis, set it to 1

sigma = 1.500;
size = 300;
x = linspace(-size / 2, size / 2, size);
gaussFilter = exp(-x .^ 2 / (2 * sigma ^ 2));
gaussFilter = gaussFilter / sum (gaussFilter); % normalize

% Data in the form compatible with MVGC
X = zeros(nvars, nobs, ntrials);

% smooth data

    cdem_f= conv(data_d(i).dem_f,gaussFilter,'same');
    cobs_f = conv(data_d(i).obs_f,gaussFilter,'same');
    
    cshock=conv(data_d(i).shock,gaussFilter,'same');

    X(1,:,i) = cdem_f(1:710);
    X(2,:,i) = cobs_f(1:710);
    X(3,:,i) = cshock(1:710);
    
%% Setting parametres

regmode   = 'OLS';  % VAR model estimation regression mode ('OLS', 'LWR' or empty for default)
icregmode = 'LWR';  % information criteria regression mode ('OLS', 'LWR' or empty for default)

morder    = 21;      %fix the order for pairwise analysis
%morder    = 'AIC';  % model order to use ('actual', 'AIC', 'BIC' or supplied numerical value)
momax     = 709;     % maximum model order for model order estimation

acmaxlags = 1000;   % maximum autocovariance lags (empty for automatic calculation)

tstat     = '';     % statistical test for MVGC:  'F' for Granger's F-test (default) or 'chi2' for Geweke's chi2 test
alpha     = 0.05;   % significance level for significance test
mhtc      = 'FDR';  % multiple hypothesis test correction (see routine 'significance')

fs        = 1;    % sample rate (Hz)
fres      = [];     % frequency resolution (empty for automatic calculation)

%%
% Calculate information criteria up to specified maximum model order.
%(Not necessary if you fixed the morder)
% 
% ptic('\n*** tsdata_to_infocrit\n');
% [AIC,BIC,moAIC,moBIC] = tsdata_to_infocrit(X,momax,icregmode);
% ptoc('*** tsdata_to_infocrit took ');
% 
% % Plot information criteria.
% 
% figure(1); clf;
% plot_tsdata([AIC BIC]',{'AIC','BIC'},1/fs);
% title('Model order estimation');
% 
% fprintf('\nbest model order (AIC) = %d\n',moAIC);
% fprintf('best model order (BIC) = %d\n',moBIC);
% 
% 
% if     strcmpi(morder,'actual')
%     morder = amo;
%     fprintf('\nusing actual model order = %d\n',morder);
% elseif strcmpi(morder,'AIC')
%     morder = moAIC;
%     fprintf('\nusing AIC best model order = %d\n',morder);
% elseif strcmpi(morder,'BIC')
%     morder = moBIC;
%     fprintf('\nusing BIC best model order = %d\n',morder);
% else
%     fprintf('\nusing specified model order = %d\n',morder);
% end
% 
% %Make histogram of orders
% pair_granger.order(i,1)= morder;
%%

% Estimate VAR model of selected order from data.

ptic('\n*** tsdata_to_var... ');
[A,SIG] = tsdata_to_var(X,morder,regmode);
ptoc;

% Check for failed regression

assert(~isbad(A),'VAR estimation failed');

% NOTE: at this point we have a model and are finished with the data! - all
% subsequent calculations work from the estimated VAR parameters A and SIG.



%%

% The autocovariance sequence drives many Granger causality calculations (see
% next section). Now we calculate the autocovariance sequence G according to the
% VAR model, to as many lags as it takes to decay to below the numerical
% tolerance level, or to acmaxlags lags if specified (i.e. non-empty).

ptic('*** var_to_autocov... ');
[G,info] = var_to_autocov(A,SIG,acmaxlags);
ptoc;

% The above routine does a LOT of error checking and issues useful diagnostics.
% If there are problems with your data (e.g. non-stationarity, colinearity,
% etc.) there's a good chance it'll show up at this point - and the diagnostics
% may supply useful information as to what went wrong. It is thus essential to
% report and check for errors here.

var_info(info,true); % report results (and bail out on error)
%%

% Calculate time-domain pairwise-conditional causalities - this just requires
% the autocovariance sequence.

ptic('*** autocov_to_pwcgc... ');
F = autocov_to_pwcgc(G);
ptoc;

% Check for failed GC calculation

assert(~isbad(F,false),'GC calculation failed');

% Significance test using theoretical null distribution, adjusting for multiple
% hypotheses.

pval = mvgc_pval(F,morder,nobs,ntrials,1,1,nvars-2,tstat); % take careful note of arguments!
sig  = significance(pval,alpha,mhtc);

% calculate and save important values for stats
a1=abs(A); %calculate absolute value of A
a2=mean(a1,3); % average A across through orders

%to save data (granger F values and other info) per dyad
pair_granger.demobsf(i,1)= F(2,1);
pair_granger.obsdemf(i,1)= F(1,2);
pair_granger.condition(i,1) =data_d(i).condition;
pair_granger.study(i,1) =data_d(i).study;
pair_granger.matID(i,1) =data_d(i).matID;

clearvars -except 'data_d' 'pair_granger' 'i'
end


